/*  $Id: MimeTypeDB.java 1188 2011-08-28 11:43:31Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2010 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 *
 * @version $Id: MimeTypeDB.java 1188 2011-08-28 11:43:31Z sdevaux $
 * @author Sebastien Devaux
 */
public class MimeTypeDB {
	/**
	 *
	 */
	public void load(String dbPath) {
		try {
			BufferedReader in=new BufferedReader(
						new InputStreamReader(new FileInputStream(dbPath)));
			String line=in.readLine();
			while(line!=null) {
				StringTokenizer st=new StringTokenizer(line);
				if (!line.trim().startsWith("#")&&st.hasMoreTokens()) {
					String mimeType=st.nextToken();
					while (st.hasMoreTokens()) {
						_typesDB.put(st.nextToken(),mimeType);
					}
				}
				line=in.readLine();
			}
			in.close();
		}
		catch (IOException ex) {
			String message="Can't read mime-type db : "+dbPath;
			if (_userdb.equals(dbPath)||_sysdb.equals(dbPath)) {
				App.get().log().info(message);
			}
			else {
				App.get().log().error(message);
			}
		}
	}
    /**
     *
     */ 
    public void loadAttr(String attrPath) {
		try {
			BufferedReader in=new BufferedReader(
						new InputStreamReader(new FileInputStream(attrPath)));
			String line=in.readLine();
			while(line!=null) {
				StringTokenizer st=new StringTokenizer(line);
				if (!line.trim().startsWith("#")&&st.hasMoreTokens()) {
					String mimeTypeMask=st.nextToken();
                    String attrName=st.nextToken();
                    String value="";
					while (st.hasMoreTokens()) {
						value+=st.nextToken()+" ";
					}
                    Hashtable<String,String> attrs=_attribDB.get(attrName);
                    if (attrs==null) {
                        // create table for this attr when not exists
                        attrs=new Hashtable<String,String>();
                        _attribDB.put(attrName,attrs);
                    }
                    attrs.put(mimeTypeMask,value);
				}
				line=in.readLine();
			}
			in.close();
		}
		catch (IOException ex) {
			SLogger log=Logger.getSLogger("net.eduvax.util");
			String message="Can't read mime-type attr db : "+attrPath;
			if (attrPath.startsWith(_userdb)||attrPath.startsWith(_sysdb)) {
				log.info(message);
			}
			else {
				log.error(message);
			}
		}
        
    }    
	
	/**
	 * get file mime type
	 * @param path file path
	 * @return file's mime-type, default is application/octet-stream
	 */
	public String getMimeType(String path) {
		int dotIndex=path.lastIndexOf('.');
		String ext="";
		if (dotIndex>=0) {
			ext=path.substring(dotIndex+1);
		}
		String res=_typesDB.get(ext);
		if (res==null) {
			return "application/octet-stream";
		}
		return res;
	}	
	/**
	 * Get attribute value matching file path.
     * @param path file path to determine mime type upon file extansion.
     * @param attrName requested attribute name.
     * @return first matching attribue value found or null if not any.
	 */
	public String getMimeAttribute(String path,String attrName) {
		Hashtable<String,String> attrs=_attribDB.get(attrName);
		if (attrs!=null) {
			Enumeration<String> keys=attrs.keys();
			boolean found=false;
			String mask=null;
			while (keys.hasMoreElements()&&!found) {
				mask=(String)keys.nextElement();
				found=getMimeType(path).matches(mask);
			}
			if (found) {
				String attr=(String)attrs.get(mask);
				// poor %s cherche and replace
				int index=attr.indexOf("%s");
				if (index>=0) {
					return attr.substring(0,index)+path+
							attr.substring(index+2);
				}
				else {
					return attr.trim();
				}	
			}
		}
		return null;
	}
	/**
	 * create new db and load defaults (sys wide and user)
	 */
	public MimeTypeDB () {
		_typesDB=new Hashtable<String,String>();
        _attribDB=new Hashtable<String,Hashtable<String,String>>();
		load(_sysdb);
        loadAttr(_sysdb+".attr");
		load(_userdb);
        loadAttr(_userdb+".attr");
	}
    
	/** types db */
	private Hashtable<String,String> _typesDB;
    /** types attributes db */
	private Hashtable<String,Hashtable<String,String>> _attribDB;
	/** default system wide DB */
	static private String _sysdb="/etc/mime.types";
	/** default user DB */
	static private String _userdb=System.getProperty("user.home")+"/.mime.types";
}
